@@ -46,9 +46,9 @@ class InstrumentBase(Metadatable, DelegateAttributes):
             Usually populated via ``add_submodule``.
     """
     def __init__(self, name: str,
-                 metadata: Optional[dict] = None) -> None:
-        self.name = str(name)
-        self.short_name = str(name)
+                 metadata: Optional[Dict] = None) -> None:
+        self._name = str(name)
+        self._short_name = str(name)
 
         self.parameters: Dict[str, _BaseParameter] = {}
         self.functions: Dict[str, Function] = {}
@@ -61,6 +61,16 @@ class InstrumentBase(Metadatable, DelegateAttributes):
 
         self.log = get_instrument_logger(self, __name__)
 
+    @property
+    def name(self) -> str:
+        """Name of the instrument"""
+        return self._name
+
+    @property
+    def short_name(self) -> str:
+        """Short name of the instrument"""
+        return self._short_name
+
     def add_parameter(self, name: str,
                       parameter_class: type = Parameter, **kwargs: Any) -> None:
         """
@@ -265,13 +275,8 @@ class InstrumentBase(Metadatable, DelegateAttributes):
             print(msg)
 
         for submodule in self.submodules.values():
-            if hasattr(submodule, '_channels'):
-                submodule = cast('ChannelList', submodule)
-                if submodule._snapshotable:
-                    for channel in submodule._channels:
-                        channel.print_readable_snapshot()
-            else:
-                submodule.print_readable_snapshot(update, max_chars)
+            submodule.print_readable_snapshot(update=update,
+                                              max_chars=max_chars)
 
     @property
     def parent(self) -> Optional['InstrumentBase']:
@@ -527,7 +532,7 @@ class Instrument(InstrumentBase, AbstractInstrument):
         if hasattr(self, 'connection') and hasattr(self.connection, 'close'):
             self.connection.close()
 
-        strip_attrs(self, whitelist=['name'])
+        strip_attrs(self, whitelist=['_name'])
         self.remove_instance(self)
 
     @classmethod
