@@ -1,6 +1,9 @@
 import textwrap
 import numpy as np
 from contextlib import ExitStack
+from functools import partial
+from typing import Sequence, Tuple
+from distutils.version import LooseVersion
 
 import qcodes.utils.validators as vals
 from qcodes import VisaInstrument, InstrumentChannel
@@ -117,7 +120,7 @@ class Trigger(InstrumentChannel):
                 it buffers one trigger.""")
         _trigger_source_vals = vals.Enum('IMM', 'EXT', 'BUS')
 
-        if self.parent.is_34465A_34470A and self.parent.has_DIG:
+        if self.parent.has_DIG:
             _trigger_source_vals = vals.Enum('IMM', 'EXT', 'BUS', 'INT')
             # extra empty lines are needed for readability of the docstring
             _trigger_source_docstring += textwrap.dedent("""\
@@ -147,9 +150,11 @@ class Sample(InstrumentChannel):
         super(Sample, self).__init__(parent, name, **kwargs)
 
         if self.parent.is_34465A_34470A:
-            _max_sample_count = 1e9
+            _max_sample_count = int(1e9)
+        elif self.parent.model == "34410A":
+            _max_sample_count = 50_000
         else:
-            _max_sample_count = 1e6
+            _max_sample_count = int(1e6)
 
         self.add_parameter('count',
                            label='Sample Count',
@@ -168,18 +173,29 @@ class Sample(InstrumentChannel):
             option) or 2,000,000 readings (with the MEM option)"""))
 
         if self.parent.has_DIG:
+            if self.parent.has_MEM:
+                _max_pretrig_count = int(2e6) - 1
+            else:
+                _max_pretrig_count = int(5e4) - 1
+
             self.add_parameter('pretrigger_count',
                                label='Sample Pretrigger Count',
                                set_cmd='SAMPle:COUNt:PRETrigger {}',
                                get_cmd='SAMPle:COUNt:PRETrigger?',
                                vals=vals.MultiType(
-                                   vals.Numbers(0, 2e6 - 1),
+                                   vals.Numbers(0, _max_pretrig_count),
                                    vals.Enum('MIN', 'MAX', 'DEF')),
                                get_parser=int,
                                docstring=textwrap.dedent("""\
                 Allows collection of the data being digitized the trigger.
                 Reserves memory for pretrigger samples up to the specified
-                num. of pretrigger samples."""))
+                num. of pretrigger samples.
+
+                Note that the maximum number of pretrigger counts is bounded
+                by the current number of sample counts as specified via the
+                ``sample.count`` parameter. Refer to the doc of the 
+                ``sample.count`` parameter for information on the maximum 
+                number of sample counts."""))
 
         if self.parent.is_34465A_34470A:
             self.add_parameter('source',
@@ -412,7 +428,7 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         ranges: A list of the available voltage ranges
     """
 
-    def __init__(self, name: str, address: str, silent: bool=False,
+    def __init__(self, name: str, address: str, silent: bool = False,
                  **kwargs):
         """
         Create an instance of the instrument.
@@ -434,9 +450,18 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         ####################################
         # Instrument specifications
 
-        self.has_DIG = 'DIG' in self._licenses()
+        options = self._options()
+        self.has_DIG = self.is_34465A_34470A and (
+            'DIG' in options
+            or LooseVersion('A.03') <= LooseVersion(idn['firmware'])
+        )
+        # Note that the firmware version check is still needed becuase ``_options``
+        # (the ``*OPT?`` command) returns 'DIG' option for firmware 3.0 only
+        # if it has been purchased before
+        self.has_MEM = self.is_34465A_34470A and 'MEM' in options
 
-        PLCs = {'34460A': [0.02, 0.2, 1, 10, 100],
+        PLCs = {'34410A': [0.006, 0.02, 0.06, 0.2, 1, 2, 10, 100],
+                '34460A': [0.02, 0.2, 1, 10, 100],
                 '34461A': [0.02, 0.2, 1, 10, 100],
                 '34465A': [0.02, 0.06, 0.2, 1, 10, 100],
                 '34470A': [0.02, 0.06, 0.2, 1, 10, 100]
@@ -445,14 +470,17 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
             PLCs['34465A'] = [0.001, 0.002, 0.006] + PLCs['34465A']
             PLCs['34470A'] = [0.001, 0.002, 0.006] + PLCs['34470A']
 
-        ranges = {'34460A': [10**n for n in range(-3, 9)],  # 1 m to 100 M
+        ranges = {'34410A': [10**n for n in range(3, 10)],  # 100 to 1 G
+                  '34460A': [10**n for n in range(-3, 9)],  # 1 m to 100 M
                   '34461A': [10**n for n in range(-3, 9)],  # 1 m to 100 M
                   '34465A': [10**n for n in range(-3, 10)],  # 1 m to 1 G
                   '34470A': [10**n for n in range(-3, 10)],  # 1 m to 1 G
                   }
 
         # The resolution factor order matches the order of PLCs
-        res_factors = {'34460A': [300e-6, 100e-6, 30e-6, 10e-6, 3e-6],
+        res_factors = {'34410A': [30e-6, 15e-5, 6e-6, 3e-6, 1.5e-6, 0.7e-6,
+                                  0.3e-6, 0.2e-6, 0.1e-6, 0.03e-6],
+                       '34460A': [300e-6, 100e-6, 30e-6, 10e-6, 3e-6],
                        '34461A': [100e-6, 10e-6, 3e-6, 1e-6, 0.3e-6],
                        '34465A': [3e-6, 1.5e-6, 0.7e-6, 0.3e-6, 0.1e-6,
                                   0.03e-6],
@@ -636,10 +664,37 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         # Measuring parameter
 
         self.add_parameter('volt',
-                           get_cmd=self._get_voltage,
+                           get_cmd=partial(self._get_parameter, "DC Voltage"),
                            label='Voltage',
                            unit='V')
 
+        self.add_parameter('curr',
+                           get_cmd=partial(self._get_parameter, "DC Current"),
+                           label='Current',
+                           unit='A')
+
+        self.add_parameter('ac_volt',
+                           get_cmd=partial(self._get_parameter, "AC Voltage"),
+                           label='AC Voltage',
+                           unit='V')
+
+        self.add_parameter('ac_curr',
+                           get_cmd=partial(self._get_parameter, "AC Current"),
+                           label='AC Current',
+                           unit='A')
+
+        self.add_parameter('res',
+                           get_cmd=partial(self._get_parameter,
+                                           "2 Wire Resistance"),
+                           label='Resistance',
+                           unit='Ohms')
+
+        self.add_parameter('four_wire_res',
+                           get_cmd=partial(self._get_parameter,
+                                           "4 Wire Resistance"),
+                           label='Resistance',
+                           unit='Ohms')
+
         #####################################
         # Time trace parameters
 
@@ -710,21 +765,50 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         """
         self.write('ABORt')
 
-    def _licenses(self):
-        licenses_raw = self.ask('SYST:LIC:CAT?')
-        licenses_list = [x.strip('"') for x in licenses_raw.split(',')]
-        return licenses_list
+    def _licenses(self) -> Sequence[str]:
+        """
+        Return extra licenses purchased with the DMM. The 34410A does not have
+        optional modules, hence always returns an empty tuple.
+        """
+        if self.model != '34410A':
+            licenses_raw = self.ask('SYST:LIC:CAT?')
+            licenses_list = [x.strip('"') for x in licenses_raw.split(',')]
+            return licenses_list
+        return tuple()
 
-    def _get_voltage(self):
-        # TODO: do we need to set any other parameters here?
+    def _options(self) -> Tuple[str, ...]:
+        """
+        Return enabled options of the DMM returned by ``*OPT?`` command.
+        The 34410A model does not have options, hence always returns an empty tuple.
+        
+        Note that for firmware version 3.0, output of ```*OPT?`` will contain
+        the ``DIG`` option only if it has been purchased before, although the option
+        itself is enabled by default in the firmware version 3.0.
+        """
+        if self.model != '34410A':
+            options_raw = self.ask('*OPT?')
+            options_list = [opt for opt in options_raw.split(',') if opt != '0']
+            return tuple(options_list)
+        return tuple()
 
-        with self.sense_function.set_to('DC Voltage'):
+    def _get_parameter(self, sense_function: str = "DC Voltage") -> float:
+        """
+        Measure the parameter given by sense_function
+
+        Args:
+            sense_function: The parameter to measure. Valid values are those
+                accepted by the sense_function parameter.
+
+        Returns:
+            The float value of the parameter.
+        """
+        with self.sense_function.set_to(sense_function):
             with self.sample.count.set_to(1):
                 response = self.ask('READ?')
 
         return float(response)
 
-    def fetch(self) -> np.array:
+    def fetch(self) -> np.ndarray:
         """
         Waits for measurements to complete and copies all available
         measurements to the instrument's output buffer. The readings remain
@@ -740,7 +824,7 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         raw_vals: str = self.ask('FETCH?')
         return _raw_vals_to_array(raw_vals)
 
-    def read(self) -> np.array:
+    def read(self) -> np.ndarray:
         """
         Starts a new set of measurements, waits for all measurements to
         complete, and transfers all available measurements.
@@ -754,13 +838,13 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         raw_vals: str = self.ask('READ?')
         return _raw_vals_to_array(raw_vals)
 
-    def _set_apt_time(self, value):
+    def _set_apt_time(self, value: float) -> None:
         self.write('SENSe:VOLTage:DC:APERture {:f}'.format(value))
 
         # setting aperture time switches aperture mode ON
         self.aperture_mode.get()
 
-    def _set_NPLC(self, value):
+    def _set_NPLC(self, value: float) -> None:
         self.write('SENSe:VOLTage:DC:NPLC {:f}'.format(value))
 
         # resolution settings change with NPLC
@@ -770,14 +854,14 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         if self.is_34465A_34470A:
             self.aperture_mode.get()
 
-    def _set_range(self, value):
+    def _set_range(self, value: float):
         self.write('SENSe:VOLTage:DC:RANGe {:f}'.format(value))
 
         # resolution settings change with range
 
         self.resolution.get()
 
-    def _set_resolution(self, value):
+    def _set_resolution(self, value: float) -> None:
         rang = self.range.get()
 
         # convert both value*range and the resolution factors
@@ -808,7 +892,7 @@ class _Keysight_344xxA(KeysightErrorQueueMixin, VisaInstrument):
         self.range.get()
 
 
-def _raw_vals_to_array(raw_vals: str) -> np.array:
+def _raw_vals_to_array(raw_vals: str) -> np.ndarray:
     """
     Helper function that converts comma-delimited string of floating-point
     values to a numpy 1D array of them. Most data retrieval command of these
